<?php
namespace Ep\Gath\Plugin\Order;

use Ep\Gath\Plugin\AbstractGathPlugin;
use WC_Order;
use WC_Order_Item;

/**
 * Bilježi dodatne informacije o cijenama u narudžbu.
 *
 * @package Ep\Gath\Plugin\Order
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class PriceInfo extends AbstractGathPlugin
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_action('woocommerce_checkout_order_processed', [$this, 'wc_checkout_order_processed'], 10, 3);

        if (defined('WP_ADMIN')) {
            add_filter('woocommerce_hidden_order_itemmeta', [$this, 'wc_hidden_order_itemmeta']);
            add_action('woocommerce_admin_order_item_headers', [$this, 'wc_admin_order_item_headers']);
        }
    }

    /**
     * U stavke narudžbe dodaje cijenu i postotak popusta/rabata.
     *
     * @param $order_id
     * @param $posted_data
     * @param WC_Order $order
     */
    public function wc_checkout_order_processed($order_id, /** @noinspection PhpUnusedParameterInspection */ $posted_data, WC_Order $order)
    {
        update_post_meta($order_id, '_order_type', $this->gath->isWholesale() ? 'wholesale' : 'retail');

        /** @var WC_Order_Item[] $items */
        $items = $order->get_items();

        foreach ($items as $item) {
            $product_id = absint(wc_get_order_item_meta($item->get_id(), '_variation_id', true));

            if (0 === $product_id) {
                $product_id = absint(wc_get_order_item_meta($item->get_id(), '_product_id', true));
            }

            if ($this->gath->isWholesale()) {
                $discountPercentage = get_post_meta($product_id, '_rebate_percentage', true);
                $discountRegularPrice = get_post_meta($product_id, '_net_price', true);
            } else {
                $discountPercentage = get_post_meta($product_id, '_sale_percentage', true);
                $discountRegularPrice = get_post_meta($product_id, '_regular_price_no_tax', true);
            }

            wc_update_order_item_meta($item->get_id(), '_discount_percentage', floatval($discountPercentage));
            wc_update_order_item_meta($item->get_id(), '_discount_regular_price', floatval($discountRegularPrice));
        }
    }

    /**
     * Spriječava naše podatke da se prikažu u standardnoj listi meta podataka.
     *
     * @param array $meta_keys
     *
     * @return array
     */
    public function wc_hidden_order_itemmeta(array $meta_keys)
    {
        $meta_keys[] = '_discount_percentage';
        $meta_keys[] = '_discount_regular_price';

        return $meta_keys;
    }

    /**
     * Zaglavlja tablice sa stavkama narudžbe.
     */
    public function wc_admin_order_item_headers()
    {
        ?>
        <th class="regular_price">Cijena bez popusta/rabata</th>
        <th class="discount">Popust/rabat</th>
        <?php
    }

    /**
     * Ispisuje dodatne vrijednosti stavke u narudžbi.
     *
     * @param $product
     * @param $item
     * @param $item_id
     */
    public function wc_admin_order_item_values(/** @noinspection PhpUnusedParameterInspection */ $product, $item, $item_id)
    {
        ?>
        <td><?= wc_price(wc_get_order_item_meta($item_id, '_discount_regular_price', true)); ?></td>
        <td><?= intval(wc_get_order_item_meta($item_id, '_discount_percentage', true)); ?>%</td>
        <?php
    }
}